use anyhow::{anyhow, Result};
use git2::{ErrorCode, Index, IndexAddOption, Oid, Repository, Signature};
use std::path::{Path, PathBuf};
use tracing::info;

pub mod history;

#[derive(Clone)]
pub struct ThreadSafeRepository {
    pub path: PathBuf,
    pub config: GitConfig,
}

#[derive(Clone)]
pub struct GitConfig {
    pub author_name: String,
    pub author_email: String,
}

impl ThreadSafeRepository {
    pub fn ensure_repository(config: GitConfig, path: &Path) -> Result<Self> {
        let res = Repository::open(path);

        match res {
            Ok(_) => Ok(Self {
                path: path.to_path_buf(),
                config,
            }),
            Err(err) => match err.code() {
                ErrorCode::NotFound => ThreadSafeRepository::create_repository(config, path),
                _ => Err(anyhow!(err)),
            },
        }
    }

    fn create_repository(config: GitConfig, path: &Path) -> Result<Self> {
        // Create repository
        let repo = Repository::init(path)?;

        // Commit all existing files
        let mut index = repo.index()?;
        index.add_all(["*/*.nix"].iter(), IndexAddOption::DEFAULT, None)?;
        let oid = index.write_tree()?;
        let tree = repo.find_tree(oid)?;
        index.write()?;

        let signature = Signature::now(&config.author_name, &config.author_email)?;
        let commit_id = repo.commit(
            Some("HEAD"),
            &signature,
            &signature,
            "Initializing stack repository",
            &tree,
            &[],
        )?;
        drop(tree);

        info!(
            commit_id = commit_id.to_string(),
            "Repository initialized with base commit"
        );

        Ok(Self {
            path: path.to_path_buf(),
            config,
        })
    }

    fn repository(&self) -> Result<Repository, git2::Error> {
        Repository::open(&self.path)
    }

    fn commit(&self, repository: Repository, mut index: Index, message: &str) -> Result<Oid> {
        index.write()?;

        let oid = index.write_tree()?;
        let tree = repository.find_tree(oid)?;
        let head = repository.head()?;

        let signature = Signature::now(&self.config.author_name, &self.config.author_email)?;
        let oid = repository.commit(
            Some("HEAD"),
            &signature,
            &signature,
            message,
            &tree,
            &[&head.peel_to_commit()?],
        )?;
        drop(tree);

        Ok(oid)
    }

    pub fn commit_files<P: AsRef<Path>>(&self, paths: &[P], message: &str) -> Result<()> {
        let repository = self.repository()?;

        // Commit file
        let mut index = repository.index()?;
        for path in paths {
            index.add_path(path.as_ref())?;
        }

        self.commit(repository, index, message)?;

        Ok(())
    }

    pub fn remove_folder(&self, path: &Path, message: &str) -> Result<()> {
        let repository = self.repository()?;

        // Commit file
        let mut index = repository.index()?;
        index.remove_dir(path, 0)?;

        self.commit(repository, index, message)?;

        Ok(())
    }
}
