use anyhow::{anyhow, Result};
use rnix::ast::Entry;
use rnix::{ast::Expr, ast::HasEntry};
use std::collections::HashMap;

pub type FlattenedSet = HashMap<String, String>;

pub fn flatten_set(expr: Expr) -> Result<FlattenedSet> {
    let set = match expr {
        Expr::AttrSet(set) => set,
        _ => return Err(anyhow!("not a set")),
    };

    let mut hashmap = HashMap::new();

    for entry in set.entries() {
        match entry {
            Entry::AttrpathValue(val) => {
                let path = val
                    .attrpath()
                    .ok_or_else(|| anyhow!("could not read attr"))?;
                let attr = path
                    .attrs()
                    .map(|ast| ast.to_string())
                    .collect::<Vec<String>>()
                    .join("\0");
                match val.value().map(flatten_set).and_then(|res| res.ok()) {
                    Some(nested_map) => {
                        for (entry, value) in nested_map {
                            hashmap.insert(format!("{}\0{}", attr, entry), value);
                        }
                    }
                    None => {
                        hashmap
                            .insert(attr, val.value().map(|v| v.to_string()).unwrap_or_default());
                    }
                }
            }
            _ => {}
        }
    }

    Ok(hashmap)
}

#[cfg(test)]
mod tests {
    use super::*;
    use rnix::Root;

    #[test]
    fn test_flatten_set_simple() {
        let test_obj = r#"{
            foo = {
                bar = "baz";
            };
        }"#;
        let ast = Root::parse(test_obj).ok().expect("invalid nix source");
        let expr = ast.expr().expect("invalid nix root");
        let flattened = flatten_set(expr).unwrap();

        assert_eq!(flattened.get("foo\0bar").unwrap(), r#""baz""#);
    }

    #[test]
    fn test_flatten_set_multiple_levels() {
        let test_obj = r#"{
            foo = {
                bar = {
                    baz = "qux";
                };
                bar.qux = "quux";
            };
            foo.boo = {
                bar = {
                    bax = "qxx";
                };
            };
            foo.fii.fuu = 123;
        }"#;

        let ast = Root::parse(test_obj).ok().expect("invalid nix source");
        let expr = ast.expr().expect("invalid nix root");
        let flattened = flatten_set(expr).unwrap();

        assert_eq!(flattened.get("foo\0bar\0baz").unwrap(), r#""qux""#);
        assert_eq!(flattened.get("foo\0bar\0qux").unwrap(), r#""quux""#);
        assert_eq!(flattened.get("foo\0boo\0bar\0bax").unwrap(), r#""qxx""#);
        assert_eq!(flattened.get("foo\0fii\0fuu").unwrap(), r#"123"#);
    }
}
