use serde::Serialize;
use sysinfo::{DiskExt, NetworkExt, NetworksExt, RefreshKind, System, SystemExt};

#[derive(Serialize)]
pub struct NetworkInterface {
    pub name: String,
    pub received: u64,
    pub transmitted: u64,
}

#[derive(Serialize)]
pub struct CpuInfo {
    pub name: String,
    pub usage: f32,
}

#[derive(Serialize)]
pub struct DiskInfo {
    pub device_name: String,
    pub mount_point: String,
    pub total_space: u64,
    pub available_space: u64,
}

#[derive(Serialize)]
pub struct SystemInfo {
    // Node info
    pub name: String,
    pub host_name: String,
    pub os_version: String,
    pub kernel_version: String,

    // RAM usage
    pub total_memory: u64,
    pub used_memory: u64,
    pub total_swap: u64,
    pub used_swap: u64,

    // Multi-elem resources
    pub networks: Vec<NetworkInterface>,
    pub disks: Vec<DiskInfo>,

    // CPU
    pub load_average: (f64, f64, f64),
}

pub fn system_info() -> SystemInfo {
    let refresh_kind: RefreshKind = RefreshKind::new().with_disks_list().with_memory();
    let mut sys = System::new_with_specifics(refresh_kind);
    sys.refresh_specifics(refresh_kind);

    let networks = sys
        .networks()
        .iter()
        .map(|(name, data)| NetworkInterface {
            name: name.clone(),
            received: data.received(),
            transmitted: data.transmitted(),
        })
        .collect();

    let avg = sys.load_average();

    let disks = sys
        .disks()
        .iter()
        .filter(|disk| !disk.is_removable() && !disk.name().eq_ignore_ascii_case("overlay"))
        .map(|disk| DiskInfo {
            device_name: disk.name().to_str().unwrap_or_default().to_string(),
            mount_point: disk.mount_point().to_str().unwrap_or_default().to_string(),
            total_space: disk.total_space(),
            available_space: disk.available_space(),
        })
        .collect();

    SystemInfo {
        name: sys.name().unwrap_or_default(),
        host_name: sys.host_name().unwrap_or_default(),
        os_version: sys.os_version().unwrap_or_default(),
        kernel_version: sys.kernel_version().unwrap_or_default(),
        total_memory: sys.total_memory(),
        used_memory: sys.used_memory(),
        total_swap: sys.total_swap(),
        used_swap: sys.used_swap(),
        networks,
        disks,
        load_average: (avg.one, avg.five, avg.fifteen),
    }
}
